import 'package:dio/dio.dart';
import 'package:flutter/material.dart';

/// BagelDB is a content management system with rich API features.
/// The way the system works is you have dedicated tokens that can be used to perform CRUD operations.
/// The access for each of those tokens can be set via your interface, in the settings option in the project page.

/// BagelDB class build a object from a specific project so that you can query
class BagelDB {
  String organizationId, apiKey, projectId;
  BagelDB({@required this.organizationId, @required this.apiKey, @required this.projectId});
}

/// allows you to build a request from [BagelDB] object
class BagelDBRequest {
  BagelDB bagelDB;
  String collection, doc, sort, order, pageNum, resPerPage;
  List<String> query = [];
  bool callEverything;
  final String baseEndpoint = 'https://devildb-gateway-gvwk7ffjaa-uc.a.run.app/api/public';
  Dio dio = Dio();

  BagelDBRequest(
      {@required this.bagelDB,
      @required this.collection,
      this.doc,
      this.sort,
      this.order,
      this.resPerPage = '10',
      this.callEverything = false});

  Future<Response> get() async {
    String queryTerm = (this.query.isNotEmpty) ? '&query=' + this.query.join("+") : '';
    String item = this.doc != null ? '/' + this.doc : '';
    String sortBy = this.sort != null ? '&sort=' + this.sort : '';
    String sortOrder = this.order != null ? '&order=' + this.order : '';
    String callEverything = this.callEverything ? '&everything=${this.callEverything}' : '';
    String pageNumber = this.pageNum != null ? '?pageNumber=${this.pageNum}' : '';
    String url =
        '$baseEndpoint/organization/${this.bagelDB.organizationId}/project/${this.bagelDB.projectId}/collection/${this.collection}/items$item?$pageNumber&perPage=${this.resPerPage}&key=${this.bagelDB.apiKey}$callEverything$queryTerm$sortBy$sortOrder';
    Response res = await dio.get(url);
    return res;
  }

  Future<Response> post({@required Map<String, dynamic> item}) async {
    String url =
        '$baseEndpoint/organization/${this.bagelDB.organizationId}/project/${this.bagelDB.projectId}/collection/${this.collection}/items';
    Response res = await dio.post(url, data: item, queryParameters: {'key': this.bagelDB.apiKey});
    return res;
  }

  Future<Response> put({@required Map<String, dynamic> item, @required String itemId}) async {
    String url =
        '$baseEndpoint/organization/${this.bagelDB.organizationId}/project/${this.bagelDB.projectId}/collection/${this.collection}/items/$itemId';
    Response res = await dio.put(url, data: item, queryParameters: {'key': this.bagelDB.apiKey});
    return res;
  }

  Future<Response> delete({@required String itemId}) async {
    String url =
        '$baseEndpoint/organization/${this.bagelDB.organizationId}/project/${this.bagelDB.projectId}/collection/${this.collection}/items/$itemId';
    Response res = await dio.delete(url, queryParameters: {'key': this.bagelDB.apiKey});
    return res;
  }

  addQuery(String queryTerm) {
    this.query.add(queryTerm);
    return this;
  }

  /// allows you to sort by a specific field
  addSort(String sortTerm) {
    this.sort = sortTerm;
    return this;
  }

  /// allows you to order, desc or asc
  addOrder(String orderTerm) {
    this.order = orderTerm;
    return this;
  }

  /// define how many results will be in each page
  addResponsesPerPage(String resultsPerPage) {
    this.resPerPage = resultsPerPage;
    return this;
  }

  /// allows you to get a specific item
  addDoc(String itemId) {
    this.doc = itemId;
    return this;
  }

  /// specify if wanted the nested objects fully
  everything() {
    this.callEverything = true;
    return this;
  }
}
